var dns = require('dns'),
    net = require('net');

exports.lookup = function (ip, provider, cb) {
  var query = this.buildQuery(ip, provider),
      self = this;

  dns.resolve(query, function (err, records) {
    var parsed = self.response(err, records);
    cb.apply(null, parsed);
  });
};

exports.response = function (err, record) {
  // If no records are returned, return null
  if (err && err.errno === 'ENOTFOUND') return [null, null];

  // Any other errors should be returned
  if (err) return [err, null];

  // If we found a record, return it
  return [null, record.pop()];
};

exports.buildQuery = function (ip, provider) {
  // Check if the ip is IPv4
  if (!net.isIPv4(ip)) {
    throw new Error('Invalid IPv4 address.');
  }

  // Reverse the IPv4 address
  var reversed = ip.split('.')
                   .reverse()
                   .join('.');

  return [reversed, provider].join('.');
};
