/*
Copyright (c) 2011, Chris Umbel

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

var ambiguous = [
    'bison', 'bream', 'carp', 'chassis', 'cod', 'corps', 'debris', 'deer',
    'diabetes', 'equipment', 'elk', 'fish', 'flounder', 'gallows', 'graffiti',
    'headquarters', 'herpes', 'highjinks', 'homework', 'information',
    'mackerel', 'mews', 'money', 'news', 'rice', 'rabies', 'salmon', 'series',
    'sheep', 'shrimp', 'species', 'swine', 'trout', 'tuna', 'whiting', 'wildebeest'
];

function addForm(singularTable, pluralTable, singular, plural) {
    singular = singular.toLowerCase();
    plural = plural.toLowerCase();
    pluralTable[singular] = plural;
    singularTable[plural] = singular;
}

function FormSet() {
    this.regularForms = [];
    this.irregularForms = {};
}

function addIrregular(singular, plural) {
    addForm(singularForms.irregularForms, pluralForms.irregularForms, singular, plural);
}

var singularForms = new FormSet();
var pluralForms = new FormSet();    

(function() {
    addIrregular("child", "children");
    addIrregular("man", "men");
    addIrregular("person", "people");
    addIrregular("sex", "sexes");
    addIrregular("mouse", "mice");
    addIrregular("ox", "oxen");
    addIrregular("foot", "feet");
    addIrregular("tooth", "teeth");
    addIrregular("goose", "geese");
    
    // see if it is possible to unify the creation of both the singular and
    // plural regexes or maybe even just have one list. with a complete list
    // of rules it may only be possible for some regular forms, but worth a shot    
    pluralForms.regularForms.push([/y$/i, 'ies']);
    pluralForms.regularForms.push([/ife$/i, 'ives']);
    pluralForms.regularForms.push([/(antenn|formul|nebul|vertebr|vit)a$/i, '$1ae']);    
    pluralForms.regularForms.push([/(octop|vir|radi|nucle|fung|cact|stimul)us$/i, '$1i']);    
    pluralForms.regularForms.push([/(buffal|tomat)o$/i, '$1oes']);    
    pluralForms.regularForms.push([/(sis)$/i, 'ses']);
    pluralForms.regularForms.push([/(matr|vert|ind)(ix|ex)$/i, '$1ices']);
    pluralForms.regularForms.push([/(x|ch|ss|sh|s|z)$/i, '$1es']);
    pluralForms.regularForms.push([/^(?!talis|.*hu)(.*)man$/i, '$1men']);
    pluralForms.regularForms.push([/(.*)/i, '$1s']);

    singularForms.regularForms.push([/([^v])ies$/i, '$1y']);
    singularForms.regularForms.push([/ives$/i, 'ife']);
    singularForms.regularForms.push([/(antenn|formul|nebul|vertebr|vit)ae$/i, '$1a']);
    singularForms.regularForms.push([/(octop|vir|radi|nucle|fung|cact|stimul)(i)$/i, '$1us']);
    singularForms.regularForms.push([/(buffal|tomat)(oes)$/i, '$1o']);
    singularForms.regularForms.push([/(analy|naly|synop|parenthe|diagno|the)ses$/i, '$1sis']);
    singularForms.regularForms.push([/(vert|ind)(ices)$/i, '$1ex']);
    // our pluralizer won''t cause this form of appendix (appendicies)
    // but we should handle it
    singularForms.regularForms.push([/(matr|append)(ices)$/i, '$1ix']);
    singularForms.regularForms.push([/(x|ch|ss|sh|s|z)es$/i, '$1']);
    singularForms.regularForms.push([/men$/i, 'man']);
    singularForms.regularForms.push([/s$/i, '']);
})();

function izeRegExps(token, forms) {
    for(i = 0; i < forms.length; i++) {
        form = forms[i];
        
        if(token.match(form[0]))
            return token.replace(form[0], form[1]);
    }
    
    return false;
}

function izeAbiguous(token) {
    if(ambiguous.indexOf(token.toLowerCase()) > -1)
        return token.toLowerCase();

    return false;
}

function izeRegulars(token, formSet) {
    token = token.toLowerCase();
    
    if(formSet.irregularForms[token])
        return formSet.irregularForms[token];

    return false;
}

function ize(token, formSet, customForms) {
    return izeRegExps(token, customForms) || izeAbiguous(token) ||
        izeRegulars(token, formSet) || izeRegExps(token, formSet.regularForms) ||
        token;
}

function attach() {
    var inflector = this;
    
    String.prototype.singularizeNoun = function() {
        return inflector.singularize(this);
    }
    
    String.prototype.pluralizeNoun = function() {
        return inflector.pluralize(this);
    }
}

var NounInflector = function() {
    this.customPluralForms = [];
    this.customSingularForms = [];

    this.addSingular = function(pattern, replacement) {
        this.customSingularForms.push([pattern, replacement]);    
    };
    
    this.addPlural = function(pattern, replacement) {
        this.customPluralForms.push([pattern, replacement]);
    };
    
    this.ize = ize;
        
    this.pluralize = function (token) {
        return this.ize(token, pluralForms, this.customPluralForms);
    };

    this.singularize = function(token) {
        return this.ize(token, singularForms, this.customSingularForms);
    };
    
    this.attach = attach;
};

module.exports = NounInflector;
