var dnsbl = require('../lib/dnsbl'),
    assert = require('assert'),
    dns = require('./stubs/dns');

describe('dnsbl', function () {

  describe('#lookup', function () {
    it('should lookup the given zone', function (done) {
      // Stub dns.resolve for testing
      dns.stubResolve(null, ['127.0.0.1']);

      var called = false;
      dnsbl.lookup('127.0.0.1', 'example.com', function (err, reply) {
        called = true;
        assert.strictEqual(reply, '127.0.0.1');
        done();
      });

      // Detect the callback was called
      setTimeout(function() {
        assert.strictEqual(called, true, 'dns.lookup callback has not been triggered');
      }, 300);

      // Restore dns.resolve
      dns.restore();
    });
  });

  describe('#buildQuery', function () {
    it('should reverse the ip and return a concat of the ip and the zone', function () {
      assert.equal(dnsbl.buildQuery('127.0.0.1', 'example.com'), '1.0.0.127.example.com');
    });
    it('should error if the ip is not a IPv4 dotted address', function () {
      assert.throws(function() {
        dnsbl.buildQuery('invalid', 'example.com');
      });
    });
  });

  describe('#response', function () {
    it('should return null if no entries are found', function () {
      var response = [{ errno: 'ENOTFOUND' }, null]
      var parsed = dnsbl.response.apply(this, response);
      assert.strictEqual(parsed[0], null);
      assert.strictEqual(parsed[1], null);
    });
    it('should parse dns response and return the reason if an entry is found', function () {
      var response = [null, ['127.0.0.2']];
      var parsed = dnsbl.response.apply(this, response);
      assert.strictEqual(parsed[0], null);
      assert.strictEqual(parsed[1], '127.0.0.2');
    });
    it('should detect timeout errors and return them', function () {
      var response = [{ errno: 'ETIMEOUT' }, null];
      var parsed = dnsbl.response.apply(this, response);
      assert.strictEqual(parsed[0].errno, 'ETIMEOUT');
      assert.strictEqual(parsed[1], null);
    });
  });

});
