var worker = require('../lib/worker'),
    dns = require('./stubs/dns'),
    assert = require('assert');

describe('worker', function () {

  describe('#process', function () {

    it('should take a job, perform the dns lookup and complete the job', function (done) {
      // Stub dns.resolve
      dns.stubResolve(null, ['127.0.0.2']);

      var job = { ip: '127.0.0.1', provider: 'example.com' };
      var callback = function (err) {
        assert.strictEqual(job.result, '127.0.0.2');
        done();
      };
      worker.process(job, callback);

      // Restore dns.resolve
      dns.restore();
    });

    it('should report nothing back if the ip is not listed', function (done) {
      // Stub dns.resolve
      dns.stubResolve({ errno: 'ENOTFOUND' });

      var job = { ip: '127.0.0.1', provider: 'example.com' };
      var callback = function (err) {
        assert.equal(job.result, null);
        done();
      };
      worker.process(job, callback);

      // Restore dns.resolve
      dns.restore();
    });

    it('should report errors back in the job if the lookup fails', function (done) {
      // Stub dns.resolve
      dns.stubResolve({ errno: 'ETIMEOUT' });

      var job = { ip: '127.0.0.1', provider: 'example.com' };
      var callback = function (err) {
        assert.strictEqual(job.error, 'ETIMEOUT');
        done();
      };
      worker.process(job, callback);

      // Restore dns.resolve
      dns.restore();
    });

  });

});
